
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"

#include <sys/types.h>                      /* time_t */
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <stdio.h>                          /* printf() */
#include <assert.h>


#include "initng_history.h"
#include "../../src/initng_active_status.h"
#include "../../src/initng_global.h"
#include "../../src/initng_toolbox.h"
#include "../../src/initng_plugin_hook.h"
#include "../ngc2/initng_ngc2.h"

int history_records = 0;

history_h history_db;

static void cmd_history(char *arg, FILE * fd);

s_command HISTORYS =
    { 'l', "history", DATA_COMMAND, USES_OPT, {(void *) &cmd_history},
"Print out history_db."
};



static void history_db_compensate_time(time_t skew)
{
    history_h *current = NULL;

    D_("history_db_compensate_time(%i);\n", (int) skew);

    while_history_db(current)
    {
        if (current->time)
            current->time += skew;
    }
}



static void history_db_clear_service(active_h * service)
{
    history_h *current = NULL;

    D_("history_db_clear_service(%s);\n", service->name);

    while_history_db(current)
    {
        if (current->service == service)
        {
            current->service = NULL;
            current->name = i_strdup(service->name);
        }
    }
}

/* clear history */
static void history_free_all(void)
{
    history_h *current, *safe = NULL;

    /* while there is a histry db */
    while_history_db_safe(current, safe)
    {
        /* free histry name */
        if (current->name)
            free(current->name);

        /* remove from history_db list */
        list_del(&current->list);

        /* free histroy entry */
        free(current);
    }
}

/* add values to history database */
static int history_add_values(active_h * service)
{
    history_h *tmp_e = NULL;    /* temporary pointer to insert data and add to db */

    assert(service->name);
    
    D_("adding: %s.\n", service->name);

    /* allocate space for data */
    if (!(tmp_e = (history_h *) initng_calloc(1, sizeof(history_h))))
    {
        F_("history_add_values(): Out of memory.\n");
        return (TRUE);
    }

    /* set data in struct */
    tmp_e->service = service;
    tmp_e->name = NULL;
    tmp_e->time = service->time_got_status;
    tmp_e->action = service->a_status;

    /* add struct */
    list_add(&tmp_e->list, &history_db.list);

    /* check lenght of history_db, and purge it on the end! */
    history_records++;

    if (history_records > HISTORY)
    {

        struct list_head *last = history_db.list.prev;
        history_h *entry = list_entry(last, history_h, list);

        if ((entry) && entry->name)
            free(entry->name);

        list_del(last);
        free(entry);
        history_records--;
    }


    /* if service is status freeing, clear the pointers in history db */
    if(service->a_status==FREEING)
    {
	history_db_clear_service(service);
    }

    /* leave */
    return (TRUE);
}


static void cmd_history(char *arg, FILE * fd)
{
    history_h *current = NULL;
    active_row row;

    while_history_db_prev(current)
    {
        if ((arg && strlen(arg) > 1)
            && !(((current->name) && strcmp(current->name, arg) == 0)
                 || ((current->service && current->service->name)
                     && strcmp(current->service->name, arg) == 0)))
            continue;

        row.s = current->action;
        row.t = current->time;
        if (current->name)
            strncpy(row.n, current->name, 100);
        else if (current->service && current->service->name)
            strncpy(row.n, current->service->name, 100);
        else
            row.n[0] = '\0';

        fwrite(&row, sizeof(active_row), 1, fd);
    }
}


int module_init(const char *version)
{
    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module, is compiled for \"%s\" version, and initng is compiled on \"%s\" version, wont load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }

    INIT_LIST_HEAD(&history_db.list);

    initng_command_add(&HISTORYS);
    initng_add_hook(ASTATUS_CHANGE, 1000, &history_add_values);
    initng_add_hook(COMPENSATE_TIME, 50, &history_db_compensate_time);

    return (TRUE);
}


void module_unload(void)
{
    initng_command_del(&HISTORYS);
    history_free_all();
    initng_del_hook(ASTATUS_CHANGE, &history_add_values);
    initng_del_hook(COMPENSATE_TIME, &history_db_compensate_time);
}
