
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#define _GNU_SOURCE
#include <fcntl.h>
#include <sys/file.h>
#include <sys/types.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <getopt.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <assert.h>


/* #include "ngc.h" */
#include "../../src/initng.h"
#include "../../src/initng_control_command.h"
#include "../../src/initng_active_status.h"
#include "initng_ngc2.h"

#include "../../src/initng_active_status.c"
/* maximum size of string rescived, warning ngc -p can contain a lot */
#define MAX_RESCIVE_SIZE 100000

#define TRUE 1
#define FALSE 0
int debug = 0;

/* Global comunication socket */
static int sock = -1;

/* functions */
void print_usage(void);
int open_socket(void);
int close_socket(void);
int send_command(const char c, const char *l, const char *o);
void command_fetch_string(void);
int command_fetch_int(void);
static int print_status(char *servicename, e_a_status status, struct tm *ts);
int main(int argc, char *argv[]);
static int fetch_result(void);


int ngc_hlp(void);
int ngc_active_db(void);

void bad_syntax(void);
char *Argv;




/* print usage on screen */
void print_usage(void)
{
    send_command('h', NULL, NULL);

}

/* open socket to communicate with */
int open_socket()
{
    int len;
    struct sockaddr_un sockname;

    /* Create the socket. */
    sock = socket(PF_UNIX, SOCK_STREAM, 0);
    if (sock < 0)
    {
        printf("Failed to init socket\n");
        return (FALSE);
    }

    /* Bind a name to the socket. */
    sockname.sun_family = AF_UNIX;
    if (debug)
        len = sprintf(sockname.sun_path, SOCKET_FILENAME_TEST);
    else
        len = sprintf(sockname.sun_path, SOCKET_FILENAME_REAL);

    len += sizeof(sockname.sun_family);

    if (connect(sock, (struct sockaddr *) &sockname, len) < 0)
    {
        close(sock);
        printf("Error connecting to socket\n");
        exit(1);
        return (FALSE);
    }
    return (TRUE);
}


/* close pipes */
int close_socket(void)
{
    close(sock);
    sock = -1;
    return (sock);
}

/* send a command */
int send_command(const char c, const char *l, const char *o)
{

    read_header header;

    /* clear structure just in case */
    memset(&header, 0, sizeof(header));

    /* fill the header with data */
    header.c = c;
    if (l)
        strncpy(header.l, l, 100);
    else
        header.l[0] = '\0';
    if (o)
        strncpy(header.o, o, 200);
    else
        header.o[0] = '\0';


    /*printf("Sending: %c, %s, %s\n", c, l ,o); */
    if ((sock < 0) && (open_socket() == FALSE))
    {
        printf("Socket is not open, or failed to open!\n");
        exit(1);
    }

    if (write(sock, &header, sizeof(read_header)) <
        (signed) sizeof(read_header))
    {
        printf("Coud not send command!\n");
        exit(1);
    }

    /* no idea to stress initng */
    usleep(50000);

    /* if we are reloading initng, we wont have an answ from it, so dont fetch any */
    if (header.c == 'c')
        return (TRUE);

    /* and of course */
    return (fetch_result());
}

/* return the fetched string */
void command_fetch_string(void)
{
    char tmp_read[MAX_RESCIVE_SIZE + 1];

    while (read(sock, &tmp_read, MAX_RESCIVE_SIZE))
        printf("%s", tmp_read);
}

/* return the fetched return number */
int command_fetch_int(void)
{
    char got_from_read[20];

    got_from_read[read(sock, &got_from_read, 19)] = '\0';
    close_socket();
    return (atoi(got_from_read));
}

static int fetch_result(void)
{
    result_desc result;

    read(sock, &result, sizeof(result));
    /*printf("got an read: command %c, status %i, type %i\n", result.c,
       result.s, result.t); */
    if (result.s == FALSE)
    {
        print_usage();
        exit(1);
    }

    switch (result.t)
    {
        case DATA_COMMAND:
            if (result.c == 'h')
            {
                ngc_hlp();
                return (TRUE);
            }
            if (result.c == 's')
            {
                ngc_active_db();
                return (TRUE);
            }
            if (result.c == 'l')
            {
                ngc_active_db();
                return (TRUE);
            }

            printf("unkown data!, no haldler!\n");
            bad_syntax();
            exit(1);

        case TRUE_OR_FALSE_COMMAND:
            {
                int r = command_fetch_int();

                if (r == TRUE)
                    printf(" command -%c returned SUCCESS! \n", result.c);
                else
                    printf(" command -%c returned FAILURE! \n", result.c);

                return (r);
            }
        case INT_COMMAND:
            printf("INT_COMMAND_RESULT: %i\n", command_fetch_int());
            return (TRUE);
        case PRINT_COMMAND:
        case STRING_COMMAND:
            command_fetch_string();
            return (TRUE);
        case VOID_COMMAND:
            printf("VOID_COMMAND\n");
            return (TRUE);
        case COMMAND_FAIL:
        default:
            printf("unkown data!, no haldler!\n");
            bad_syntax();
            exit(1);

    }
    printf("Error, shud not got here!\n");
    exit(22);
}

/* THIS IS MAIN */
int main(int argc, char *argv[])
{
    int cc = 1;

    Argv = argv[0];

    /* print banner */
    printf("Next Generation init Control.\n");
    printf("written by Jimmy Wennlund <jimmy.wennlund@gmail.com>\n\n");



    /* check if cmd-line contains "ngdc", if set debug */
    if (strstr(argv[0], "ngdc"))
    {
        printf("Warning. This is ngdc!\n");
        debug = 1;
    }

    /* make sure there are any arguments at all */
    if (argc < 2)
        print_usage();


    while (argv[cc])
    {
        char *opt = NULL;

        /* every fresh start needs a '-' char */
        if (argv[cc][0] != '-')
        {
            bad_syntax();
            exit(1);
        }

        /* check that there is a char after the '-' */
        if (!argv[cc][1])
        {
            bad_syntax();
            exit(1);
        }

        /* if next option contains data, but not starting with an '-', its considered an option */
        if (argv[cc + 1] && argv[cc + 1][0] != '-')
            opt = argv[cc + 1];


        /* if it is an --option */
        if (argv[cc][1] == '-')
        {
            if (!send_command('\0', &argv[cc][2], opt))
            {
                printf("Command %s, %s failed!\n", &argv[cc][2], opt);
                exit(1);
            }
            /* if it is an -o option */
        }
        else
        {
            if (!send_command(argv[cc][1], NULL, opt))
            {
                printf("Command %c, %s failed!\n", argv[cc][1], opt);
                exit(1);
            }
        }

        cc++;
        if (opt)
            cc++;


    }
    exit(0);
}


static int print_status(char *sname, e_a_status status, struct tm *ts)
{
    char servicename[41];
    int i;

    /* copy name to the new static array */
    strncpy(servicename, sname, 40);

    /* Fill out 40 blanks for nice output */
    for (i = strlen(servicename); i <= 40; i++)
        servicename[i] = ' ';

    /* to make sure, there is an end */
    servicename[40] = '\0';

    printf(" %.2i:%.2i:%.2i %s : %s\n", ts->tm_hour, ts->tm_min,
                   ts->tm_sec, servicename, active_db_get_status_string(status));

    return (TRUE);

}

#ifdef NO
/* get history */
int ngc_history(char *service)
{
    char *got_this = NULL;
    int i = 0;
    int type;
    long int t;
    struct tm *ts;
    char servicename[41];
    char write_this[101];

    /* Fetch input from initng */
    if (service)
    {
        /*printf("\"p%s\"", service); */
        sprintf(write_this, "l%s", service);
        send_command(write_this);
    }
    else
        send_command("l");

    got_this = command_fetch_string(got_this);

    if (!got_this)
        return (FALSE);

    if (strcmp(got_this, "0\n") == 0)
    {
        while ((got_this = command_fetch_string(got_this)))
            printf("%s", got_this);
        return (FALSE);
    }

    /* print head */
    printf(" hh:mm:ss service                                  : status\n");
    printf(" ----------------------------------------------------------\n");

    while (got_this)
    {
        /* Parse the line into type and servicename */
        if (!sscanf(got_this, "%i|%li|%s\n", &type, &t, servicename))
            break;

        time_t test = (time_t) t;

        ts = localtime(&test);

        /*timen=(time_t)t; */

        /* Fill out 40 blanks for nice output */
        for (i = strlen(servicename); i <= 40; i++)
            servicename[i] = ' ';
        servicename[40] = '\0';

        print_status(servicename, type, ts);

        got_this = command_fetch_string(got_this);
    }
    return (TRUE);
}
#endif

/* get help */
int ngc_hlp(void)
{
    /* data got */
    help_row row;

    /* print head */
    printf(" ngc understand this commands:\n");
    printf(" option                          : description\n");
    printf(" ----------------------------------------------------------\n");

    while (read(sock, &row, sizeof(help_row)))

    {
        int i;
        char lname[26];

        /* copy name to the new static array */
        strncpy(lname, row.l, 25);

        switch (row.o)
        {
            case USES_OPT:
                strcat(lname, " <opt>");
                break;
            case REQUIRES_OPT:
                strcat(lname, " opt");
                break;
            default:
                break;
        }

        /* Fill out 40 blanks for nice output */
        for (i = strlen(lname); i <= 25; i++)
            lname[i] = ' ';

        lname[25] = '\0';
	if(row.c!='\0')
    	    printf(" [-%c] --%s: %s\n", row.c, lname, row.d);
	else
    	    printf("       --%s: %s\n", lname, row.d);
    }
    return (TRUE);
}

int ngc_active_db(void)
{
    active_row row;

    /* print head */
    printf(" hh:mm:ss service                                  : status\n");
    printf(" ----------------------------------------------------------\n");

    while (read(sock, &row, sizeof(active_row)))
    {
        print_status(row.n, row.s, localtime(&row.t));

    }
    return (TRUE);
}

#ifdef  NO
/* get running processes */
int ngc_status(char *service)
{
    char *got_this = NULL;
    int i = 0;
    long int ti;
    struct tm *ts;
    int type;
    char servicename[41];
    char write_this[101];

    /* Fetch input from initng */
    if (service)
    {
        /*printf("\"p%s\"", service); */
        sprintf(write_this, "s%s", service);
        send_command(write_this);
    }
    else
        send_command("s");

    got_this = command_fetch_string(got_this);
    if (!got_this)
        return (FALSE);

    if (strcmp(got_this, "0\n") == 0)
    {
        while ((got_this = command_fetch_string(got_this)))
            printf("%s", got_this);
        return (FALSE);
    }

    /* print head */
    printf(" hh:mm:ss service                                  : status\n");
    printf(" ----------------------------------------------------------\n");

    while (got_this)
    {
        /* Parse the line into type and servicename */
        if (!sscanf(got_this, "%i|%li|%s\n", &type, &ti, servicename))
            break;

        time_t test = (time_t) ti;

        ts = localtime(&test);

        /* Fill out 40 blanks for nice output */
        for (i = strlen(servicename); i <= 40; i++)
            servicename[i] = ' ';
        servicename[40] = '\0';

        print_status(servicename, type, ts);

        got_this = command_fetch_string(got_this);
    }
    return (TRUE);
}
#endif

#ifdef NO
/* restart initng */
int ngc_toggle_verbose(char *opt)
{
    if (opt)
        return (0);
    int a = command_fetch_int("v");

    switch (a)
    {
        case 0:
            printf("Verbose is off, and won't watch g.verbose_this\n");
            break;
        case 1:
            printf("All verbose is on\n");
            break;
        case 2:
            printf("Will verbose whats in g.verbose_this\n");
            break;
        case 3:
            printf("Will verbose all but ! in g.verbose_this\n");
            break;
        default:
            printf("Unknown verbose mode!\n");
            return (FALSE);
            break;
    }
    return (TRUE);
}
#endif

/* Avoid using goto */
void bad_syntax(void)
{
    printf("Bad syntax!\n\n");
    print_usage();
}
