
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "../../src/initng.h"
#include "../../src/initng_handler.h"

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

#include "../../src/initng_global.h"
#include "../../src/initng_string_tools.h"
#include "../../src/initng_active_db.h"
#include "../../src/main.h"
#include "../../src/initng_plugin_hook.h"
#include "../../src/initng_toolbox.h"
#include "initng_splash.h"

int print_progress(active_h * service)
{
    if (!service)
        return (TRUE);

    int percent = 0;
    FILE *fd = NULL;

    s_entry *virtual = initng_sdt_find("virtual");

    /* skip virtual services */
    if ((virtual) && active_db_is(virtual, service))
        return (TRUE);

    switch (g.sys_state)
    {
        case STATE_STARTING:
        case STATE_UP:
            if (((service->a_status != DONE)
                 && (service->a_status != RUNNING))
                || (service->from_service->type == TYPE_RUNLEVEL))
                return (TRUE);
            percent = active_db_percent_started();
            break;
        case STATE_STOPPING:
        case STATE_RESTART:
        case STATE_EXIT:
        case STATE_ASE:
            if ((service->a_status != STOPPED)
                && (service->a_status != RESTARTING))
                return (TRUE);
            percent = 100 - active_db_percent_stopped();
            break;
        default:
            break;
    }


    /* open splash control file */
    /* Warn here? */
    if (!(fd = fopen(SPLASH_CONTROL_FILE, "w")))
        return (TRUE);

    fprintf(fd, "show %d\n", (65535 * percent) / 100);

    fclose(fd);
    return (TRUE);
}

void print_system_state(h_sys_state state)
{

    int pos = 0;
    int cmdline = 0;
    struct stat st;
    char *buffer;
    char mode[10];

    FILE *fd = NULL;

    cmdline = open("/proc/cmdline", O_RDONLY);

    /* Check if splash contorl file exists */
    if ((cmdline < 0) || (fstat(cmdline, &st) < 0))
        return;

    /* Make a buffer and read entire file */
    buffer = initng_calloc(st.st_size + 1, 1);

    pos = read(cmdline, buffer, st.st_size);
    if (pos != st.st_size)
    {
        F_("Failed to read file /proc/cmdline\n");
        free(buffer);
        close(cmdline);
        return;
    }

    close(cmdline);

    if (!strstr(buffer, "splash=silent"))
    {
        free(buffer);
        return;
    }

    free(buffer);

    /* open splash control file */
    /* Warn here? */
    if (!(fd = fopen(SPLASH_CONTROL_FILE, "w")))
        return;

    switch (state)
    {
        case STATE_UP:
            strcpy(mode, "master");
            break;
        case STATE_STOPPING:
        case STATE_RESTART:
        case STATE_EXIT:
        case STATE_ASE:
            strcpy(mode, "silent");
            break;
        default:
            break;
    }

    fprintf(fd, "%s\n", mode);

    fclose(fd);
}

int module_init(const char *version)
{
    D_("splash: module_load();\n");
    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module, is compiled for \"%s\" version, and initng is compiled on \"%s\" version, wont load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }

    initng_add_hook(WATCHERS, 80, &print_progress);
    initng_add_hook(SWATCHERS, 80, &print_system_state);
    return (TRUE);
}

void module_unload(void)
{
    D_("splash: module_unload();\n");
    initng_del_hook(WATCHERS, &print_progress);
    initng_del_hook(SWATCHERS, &print_system_state);
}
