
/* Initng, a 6next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"

#define _GNU_SOURCE
#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <time.h>

#include "../../src/initng_toolbox.h"
#include "../../src/initng_global.h"
#include "../../src/initng_service_cache.h"
#include "../../src/initng_active_db.h"


#include "print_service.h"

static void service_db_print_u(service_h * s, FILE * fd, int intro);
static void active_db_print_u(active_h * s, FILE * fd, int intro);

/* P R I N T   S E R V I C E */

/* some stdout */
void service_db_print(service_h * s, FILE * fd)
{
    service_db_print_u(s, fd, TRUE);
    return;
}

void active_db_print(active_h * s, FILE * fd)
{
    active_db_print_u(s, fd, TRUE);
    return;
}

static void service_db_print_u(service_h * s, FILE * fd, int intro)
{
    /*data path */
    s_data *tmp = NULL;

    assert(s);
    assert(s->name);

    /* try calculate out how mutch memory we wanna allocate
       that we will send */

    if (intro)
        switch (s->type)
        {
            case TYPE_SERVICE:
                fprintf(fd, "\n#  service  \"%s", s->name);
                if (s->father_name)
                    fprintf(fd, " : %s", s->father_name);
                fprintf(fd, "\"\n");
                break;
            case TYPE_CLASS:
                fprintf(fd, "\n#  class    \"%s\"\n", s->name);
                break;
            case TYPE_RUNLEVEL:
                fprintf(fd, "\n#  runlevel \"%s\"\n", s->name);
                break;
	    case TYPE_VIRTUAL:
                fprintf(fd, "\n#  virtual \"%s\"\n", s->name);
                break;	    
            default:
                fprintf(fd, "\n#  unknown  \"%s\"\n", s->name);
                break;
        }

    fprintf(fd, "    ------------------------ \n");

    list_for_each_entry(tmp, &s->data.list, list)
    {
        if (tmp->type)
        {
            if (tmp->type->opt_type == STRING ||
                tmp->type->opt_type == STRINGS
                || tmp->type->opt_type == SCRIPT)
            {

                if (!tmp->t.s)
                {
                    F_("empty value!\n");
                }
                else
                {
                    fprintf(fd, " @@ %s:\t\"%s\"\n", tmp->type->opt_name,
                            tmp->t.s);
                }
            }
            else if (tmp->type->opt_type == INT)
            {
                fprintf(fd, " @@ %s:\t\"%i\"\n", tmp->type->opt_name,
                        tmp->t.i);
            }
            else if (tmp->type->opt_type == SET)
            {
                fprintf(fd, " @@ %s\n", tmp->type->opt_name);
            }
        }
    }

    /* if father with data exits, print it */
    if (s->father)
    {
        D_("FATHER FOUND %s\n", s->name);
        service_db_print_u(s->father, fd, FALSE);
    }
    else
        D_("FATHER NOT FOUND %s\n", s->name);

    return;
}

static void print_max_20(char *from, FILE * fd)
{
    int i=0;
    char to_pr[50];
    while(from[0]==' ' || from[0]=='\n' || from[0]=='\t') from++;
    while(i<=40 && from[i] && from[i]!='\n') i++;
    strncpy(to_pr, from, i);
    to_pr[i]='\0';
    if(i>=40)
	fprintf(fd, "%s ...", to_pr);
    else
	fprintf(fd, "%s", to_pr );
}

/* Walk through every service and print it all */
void service_db_print_all(FILE * fd)
{

    service_h *current = NULL;

    fprintf(fd, "Full service print:\n");

    D_("print_all():\n");

    while_service_db(current)
    {
        D_("printing %s ...\n", current->name);
        service_db_print(current, fd);
    }

    return;
}
static void active_db_print_process(process_h * p, FILE * fd)
{
    assert(p);
    fprintf(fd, "\tProcess: type %i\n", p->pt);
    fprintf(fd, "\t\tPid: %i\n", p->pid);
    fprintf(fd, "\t\tR_code: %i\n", p->r_code);
    fprintf(fd, "\t\tFds read: %i, write: %i\n", p->stdout[0], p->stdout[1]);
    if(p->buffer) {
	fprintf(fd, "\t\tBuffer: \"");
	print_max_20(p->buffer, fd);
	fprintf(fd, "\"\n");
    }
    fprintf(fd, "\t\tBuffer-pos: %i\n\n", p->buffer_pos);

}

static void active_db_print_u(active_h * s, FILE * fd, int intro)
{
    /*data path */
    s_data *tmp = NULL;

    assert(s);
    assert(s->name);

    /* try calculate out how mutch memory we wanna allocate
       that we will send */

    if (intro)
    {
            switch (s->type)
            {
                case TYPE_SERVICE:
                    fprintf(fd, "\n#  service  \"%s", s->name);
                    if (s->from_service && s->from_service->father_name)
                        fprintf(fd, " : %s", s->from_service->father_name);
                    fprintf(fd, "\"\n");
                    break;
                case TYPE_CLASS:
                    fprintf(fd, "\n#  class    \"%s\"\n", s->name);
                    break;
                case TYPE_RUNLEVEL:
                    fprintf(fd, "\n#  runlevel \"%s\"\n", s->name);
                    break;
		case TYPE_VIRTUAL:
		    fprintf(fd, "\n#  virtual \"%s\"\n", s->name);
		    break;
                default:
                    fprintf(fd, "\n#  unknown  \"%s\"\n", s->name);
                    break;
            }

	fprintf(fd, "\tactive status: %s\n",
            active_db_get_status_string(s->a_status));

	fprintf(fd, "\tm_needed: %i\n", s->m_needed);
	fprintf(fd, "\tm_rest: %i\n", s->m_rest);
	fprintf(fd, "\ton_exit: %i\n", s->on_exit);
	fprintf(fd, "\tuptime: %i time_got_status: %i\n", (int) (time(0) - s->uptime), (int) (time(0) - s->time_got_status));

	/* print processes if any */
	if(s->start_process)
	    active_db_print_process(s->start_process, fd);
	if(s->stop_process)
	    active_db_print_process(s->stop_process, fd);

    }


    

    fprintf(fd, "\tactive_db opts:\n");

    list_for_each_entry(tmp, &(s->data.list), list)
    {
        if (tmp->type)
        {
	    switch (tmp->type->opt_type)
	    {
		case STRING:
		case STRINGS:
		case SCRIPT:

            	    if (!tmp->t.s) break;
		    fprintf(fd, "\t\t@@ %s:\t\"", tmp->type->opt_name);
		    print_max_20(tmp->t.s, fd);
		    fprintf(fd, "\"\n");
		    break;
		
		case INT:
            	    fprintf(fd, "\t\t@@ %s:\t\"%i\"\n", tmp->type->opt_name,
                        tmp->t.i);
		    break;
		
		case SET:
		    fprintf(fd, "\t\t@@ %s\n", tmp->type->opt_name);
		    break;

		case UNSET:
		    fprintf(fd, "\t\t@@ !! %s !!\n", tmp->type->opt_name);
		    break;
		default:
		    break;

            }
        }
    }

    if (s->from_service)
    {
        fprintf(fd, "\tservice_db opts:\n");
        tmp = NULL;
        list_for_each_entry(tmp, &(s->from_service->data.list), list)
        {
            if (tmp->type)
            {
		switch (tmp->type->opt_type)
		{
		case STRING:
		case STRINGS:
		case SCRIPT:

            	    if (!tmp->t.s) break;
		    fprintf(fd, "\t\t@@ %s:\t\"", tmp->type->opt_name);
		    print_max_20(tmp->t.s, fd);
		    fprintf(fd, "\"\n");
		    break;

		case INT:
            	    fprintf(fd, "\t\t@@ %s:\t\"%i\"\n", tmp->type->opt_name,
                            tmp->t.i);
            	    break;
		case SET:
                    fprintf(fd, "\t\t@@ %s\n", tmp->type->opt_name);
		    break;
		case UNSET:
		    fprintf(fd, "\t\t@@ !! %s !!\n", tmp->type->opt_name);
		    break;
		default:
		    break;
                }
            }
        }
    }
}

/* Walk through every service and print it all */
void active_db_print_all(FILE * fd)
{

    active_h *apt = NULL;

    fprintf(fd, "Full service print:\n");

    D_("print_all():\n");

    while_active_db(apt)
    {
        active_db_print(apt, fd);
        fprintf(fd, "\n\n");
    }

    return;
}
