
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "initng.h"

#include <time.h>                           /* time() */
#include <fcntl.h>                          /* fcntl() */
#include <unistd.h>                         /* execv() pipe() usleep() pause() chown() */
#include <sys/wait.h>                       /* waitpid() sa */
#include <sys/ioctl.h>                      /* ioctl() */
#include <stdlib.h>                         /* free() exit() */
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <pwd.h>
#include <sys/stat.h>

#include "initng_global.h"
#include "initng_process_db.h"
#include "main.h"
#include "initng_string_tools.h"            /* split() */
#include "initng_execute.h"
#include "initng_active_db.h"
#include "initng_plugin_callers.h"
#include "initng_toolbox.h"
#include "initng_common.h"
#include "initng_static_data_id.h"
#include "initng_plugin_type.h"

int start_launch(active_h * service)
{
    s_call *current, *q=NULL;
    int ret=0;
    assert(service);
    D_("start_launch(%s);\n", service->name);

    if (!(service->start_process = process_db_new(T_START)))
    {
        F_("start_launch(%s): unable to allocate start_process!\n",
           service->name);
        return (FALSE);
    }

    if (!mark_service(service, START_LAUNCHING))
    {
        process_db_free(service->start_process);
        service->start_process = NULL;
        return (FAIL);
    }

    /* walk the db with hooks */
    while_call_db_safe(current,q)
    {
	if(current->ptype!=STARTL) continue;
	
	/* call the start */
	ret = (*current->c.hook) (service);
	
	/* if lauch succeded */
	if(ret>=TRUE) return(TRUE);

	/* if that launcher did not get any to launch, take next launcher */
	if(ret==FALSE) continue;
	if(ret<FALSE) break;
    }

    if(ret<FALSE)
	F_("initng_execute(%s): FAILED EXECUTING START, returned FAIL\n", service->name);
    else
	D_("initng_execute(%s): FAILED EXECUTING START, resturned FALSE\n", service->name);
	
    process_db_free(service->start_process);
    service->start_process = NULL;
    
    return(ret);
}


int stop_launch(active_h * service)
{
    s_call *current, *q=NULL;
    int ret=0;
    assert(service);
    D_("stop_launch(%s);\n", service->name);

    /* create a new stop process */
    if (!(service->stop_process = process_db_new(T_STOP)))
    {
        F_("stop_launch(%s): unable to allocate stop_process!\n",
           service->name);
        return (FALSE);
    }

    if (!mark_service(service, STOP_LAUNCHING))
    {
        process_db_free(service->stop_process);
        service->stop_process = NULL;
        return (FAIL);
    }

    /* walk the db with hooks */
    while_call_db_safe(current,q)
    {
	if(current->ptype!=STOPL) continue;
	
	/* call the start */
	ret = (*current->c.hook) (service);
	
	/* if lauch succeded */
	if(ret>=TRUE) return(TRUE);

	/* if that launcher did not get any to launch, take next launcher */
	if(ret==FALSE) continue;
	if(ret<FALSE) break;
    }

    if(ret<FALSE)
	F_("initng_execute(%s): FAILED EXECUTING START, returned FAIL\n", service->name);
    else
	D_("initng_execute(%s): FAILED EXECUTING START, resturned FALSE\n", service->name);
	
    process_db_free(service->stop_process);
    service->stop_process = NULL;
    
    return(ret);
}


/* creates a custom environment variable */
char **new_environ(active_h * s)
{
    int nr = 0;
    char **env;

    /* Better safe than sorry... */
    if (s && s->name == NULL)
        s->name = i_strdup("unknown");

    /* count existing env's */
#ifdef USE_OLD_ENV
    while (environ[nr])
        nr++;
#endif
#ifdef DEBUG
    if (s)
        D_("new_environ(): there are %i envs in config %s!\n",
           active_db_count_type(&ENV, s), s->name);
#endif
    /*  Add 11 more environs */
    if (s)
        nr = nr + 11 + active_db_count_type(&ENV, s);

    /* allocate */
    env = (char **) initng_calloc(nr, sizeof(char *));

    /* duplicate */
#ifdef USE_OLD_ENV
    for (nr = 0; environ[nr]; nr++)
    {
        env[nr] = i_strdup(environ[nr]);
    }
#else
    nr = 0;
#endif
    /* add custom */
    env[nr++] = i_strdup("INITNG=" INITNG_VERSION);
    env[nr++] =
        i_strdup
        ("PATH=/bin:/usr/bin:/sbin:/usr/sbin:/usr/local/bin:/opt/bin");
    env[nr++] = i_strdup("HOME=/root");
    env[nr++] = i_strdup("USER=root");
    env[nr++] = i_strdup("TERM=linux");
    /*env[nr++] = i_strdup("BASH_ENV=\"" INITNG_ROOT "/bash-env.sh\""); */

    if (s)
    {
        assert(s);
        env[nr] =
            (char *) initng_calloc(1, sizeof(char) * (9 + strlen(s->name)));
        strcpy(env[nr], "SERVICE=");
        strcat(env[nr], s->name);
        nr++;

        env[nr] =
            (char *) initng_calloc(1, sizeof(char) * (6 + strlen(s->name)));
        strcpy(env[nr], "NAME=");
        strcat(env[nr], st_strip_path(s->name));
        nr++;

        env[nr] =
            (char *) initng_calloc(1, sizeof(char) * (7 + strlen(s->name)));
        strcpy(env[nr], "CLASS=");
        strcat(env[nr], s->name);
        st_strip_end(&env[nr]);
        nr++;


        env[nr] = NULL;
        /* insert all env strings from config */
        if ((env[nr] = active_db_get_next_string(&ENV, s, NULL)))
        {
            nr++;
            while ((env[nr] =
                    active_db_get_next_string(&ENV, s, env[nr - 1])))
                nr++;

        }
    }
    /* null last */
    if (env[nr] != NULL)
        env[nr++] = NULL;

#ifdef DEBUG
    for (nr = 0; env[nr]; nr++)
    {
        D_("new_environ(): environ[%i] = \"%s\"\n", nr, env[nr]);
    }
#endif

    /* return new environ */
    return (env);
}

/* this frees an environment variable */
void free_environ(char **tf)
{
    int i = 0;

    D_("free_environ();\n");
    assert(tf);
    for (i = 0; tf[i]; tf++)
    {
        free(tf[i]);
    }
    free(tf);
}

/*
 * set_pipes() set up filedescriptors, for local fork,
 * a fork in initng, shud now rescive any input, but stdout & stderr, shud be sent
 * to process->stdout[], that is set up by pipe() #man 2 pipe
 * [0] for reading, and [1] for writing, as the pipe is for sending output
 * FROM the fork, to initng for handle, the input part shud be closed here,
 * the other are maped to STDOUT and STDERR.
 */
void set_pipes(process_h * process)
{
    /* close stdin/stdout/stderr */
    /*close(STDIN_FILENO);*/
    close(STDOUT_FILENO);
    close(STDERR_FILENO);

    /* Duplicate stdout and stderr to the stdout[1] */
    dup2(process->stdout[1], STDOUT_FILENO);
    dup2(process->stdout[1], STDERR_FILENO);

    /* set stdin, stdout, and stderr, that is shud not be closed, if this child do execve() */
    fcntl(STDIN_FILENO, F_SETFD, 0);
    fcntl(STDOUT_FILENO, F_SETFD, 0);
    fcntl(STDERR_FILENO, F_SETFD, 0);

    /* Close the sides of the pipes we don't need, as we're in fork we won't need this part. */
    close(process->stdout[0]);
    process->stdout[0] = -1;
}

