
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef SERVICE_DB_H
#define SERVICE_DB_H

#include "initng_struct_data.h"
#include "initng_list.h"

/* service type enum */
typedef enum
{ TYPE_UNKNOWN, TYPE_SERVICE, TYPE_CLASS, TYPE_RUNLEVEL, TYPE_VIRTUAL } e_type;

typedef struct s_service_h service_h;
struct s_service_h
{
    char *name;
    e_type type;

    /* list container, to store data */
    s_data data;

    /* father/relative data */
    char *father_name;
    service_h *father;

    /* add a list entry */
    struct list_head list;
};


/* service haneling functions */
int service_db_add(service_h * s);
service_h *service_db_copy(char *name, service_h * s);
service_h *service_db_new(char *name, e_type type);
service_h *service_db_find_by_name(char *find);
service_h *service_db_find_by_exact_name(char *find);
service_h *service_db_find_in_name(char *find);
int service_db_find_father(service_h * service);
int service_db_free(service_h * to_free);
void service_db_free_all(void);
void service_db_change(service_h * from, service_h * to);

/* standard wrappers using service */
char *service_db_get_string(s_entry * type, service_h * s);
int service_db_get_int(s_entry * type, service_h * s);
char *service_db_get_next_string(s_entry * type, service_h * s, char *last);
int service_db_is(s_entry * type, service_h * s);

/*char **service_db_get_stringlist(s_entry * type, service_h * s); */
#define service_db_set_string(type, from_service, string)  d_set_string(type, &from_service->data.list, string)
#define service_db_set_another_string(type, from_service, string) d_set_another_string(type, &from_service->data.list, string)
#define service_db_set_int(type, from_service, value) d_set_int(type, &from_service->data.list, value)
#define service_db_set(type, from_service) d_set(type, &from_service->data.list)
#define service_db_remove(type, from_service) d_remove(type, &from_service->data.list)
#define service_db_remove_all(from_service) d_remove_all(&from_service->data.list)
#define service_db_count_type(type, from_service) d_count_type(type, &from_service->data.list)

/* service_db walker */
#define while_service_db(current) list_for_each_entry(current, &g.service_cache.list, list)
#define while_service_db_safe(current, safe) list_for_each_entry_safe(current, safe, &g.service_cache.list, list)

#endif
